# Generated by roxygen2: do not edit by hand

export(FPR)
export(SCCA_Parkhomenko)
export(SparseCCA)
export(TNR)
export(TPR)
export(Witten.CV)
export(cca_graph_rrr)
export(cca_graph_rrr_cv)
export(cca_group_rrr)
export(cca_group_rrr_cv)
export(cca_rrr)
export(cca_rrr_cv)
export(ecca)
export(ecca.cv)
export(get_edge_incidence)
export(principal_angles)
export(regular_cca)
export(sinTheta)
export(sparse_CCA_benchmarks)
export(subdistance)
importFrom(foreach,"%do%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(magrittr,"%>%")
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,median)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(tidyr,replace_na)
