% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alt_regular_CCA.R
\name{regular_cca}
\alias{regular_cca}
\title{Function to perform regular (low dimensional) canonical correlation analysis (CCA}
\usage{
regular_cca(X, Y, rank)
}
\arguments{
\item{X}{Matrix of predictors (n x p)}

\item{Y}{Matrix of responses (n x q)}

\item{rank}{Number of canonical components to extract}
}
\value{
A list with elements:
\describe{
\item{U}{Canonical direction matrix for X (p x r)}
\item{V}{Canonical direction matrix for Y (q x r)}
\item{cor}{Canonical covariances}
}
}
\description{
Function to perform regular (low dimensional) canonical correlation analysis (CCA
}
